/***************************************************************************
 *   Copyright (C) 2010~2010 by CSSlayer                                   *
 *   wengxt@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
 ***************************************************************************/

#ifndef _FCITX_KEYS_H_
#define _FCITX_KEYS_H_

#include <fcitx-config/hotkey.h>

#ifdef __cplusplus
extern "C" {
#endif

    /*
     * Define const keys that will be used in code
     */

    extern FcitxHotkey FCITX_NONE_KEY[2];
    extern FcitxHotkey FCITX_DELETE[2];
    extern FcitxHotkey FCITX_CTRL_DELETE[2];
    extern FcitxHotkey FCITX_BACKSPACE[2];
    extern FcitxHotkey FCITX_HOME[2];
    extern FcitxHotkey FCITX_END[2];
    extern FcitxHotkey FCITX_RIGHT[2];
    extern FcitxHotkey FCITX_LEFT[2];
    extern FcitxHotkey FCITX_ESCAPE[2];
    extern FcitxHotkey FCITX_ENTER[2];
    extern FcitxHotkey FCITX_LCTRL_LSHIFT[2];
    extern FcitxHotkey FCITX_LCTRL_LSHIFT2[2];
    extern FcitxHotkey FCITX_RCTRL_RSHIFT[2];
    extern FcitxHotkey FCITX_RCTRL_RSHIFT2[2];
    extern FcitxHotkey FCITX_LALT_LSHIFT[2];
    extern FcitxHotkey FCITX_LALT_LSHIFT2[2];
    extern FcitxHotkey FCITX_RALT_RSHIFT[2];
    extern FcitxHotkey FCITX_RALT_RSHIFT2[2];
    extern FcitxHotkey FCITX_LCTRL_LSUPER[2];
    extern FcitxHotkey FCITX_LCTRL_LSUPER2[2];
    extern FcitxHotkey FCITX_RCTRL_RSUPER[2];
    extern FcitxHotkey FCITX_RCTRL_RSUPER2[2];
    extern FcitxHotkey FCITX_LALT_LSUPER[2];
    extern FcitxHotkey FCITX_LALT_LSUPER2[2];
    extern FcitxHotkey FCITX_RALT_RSUPER[2];
    extern FcitxHotkey FCITX_RALT_RSUPER2[2];
    extern FcitxHotkey FCITX_SEMICOLON[2];
    extern FcitxHotkey FCITX_SPACE[2];
    extern FcitxHotkey FCITX_SHIFT_SPACE[2];
    extern FcitxHotkey FCITX_SHIFT_ENTER[2];
    extern FcitxHotkey FCITX_TAB[2];
    extern FcitxHotkey FCITX_COMMA[2];
    extern FcitxHotkey FCITX_PERIOD[2];
    extern FcitxHotkey FCITX_CTRL_5[2];
    extern FcitxHotkey FCITX_SEPARATOR[2];
    extern FcitxHotkey FCITX_CTRL_ALT_E[2];
    extern FcitxHotkey FCITX_LCTRL[2];
    extern FcitxHotkey FCITX_LSHIFT[2];
    extern FcitxHotkey FCITX_RCTRL[2];
    extern FcitxHotkey FCITX_RSHIFT[2];
    extern FcitxHotkey FCITX_ALT_LSHIFT[2];
    extern FcitxHotkey FCITX_ALT_RSHIFT[2];
    extern FcitxHotkey FCITX_CTRL_LSUPER[2];
    extern FcitxHotkey FCITX_CTRL_RSUPER[2];
    extern FcitxHotkey FCITX_SUPER_LCTRL[2];
    extern FcitxHotkey FCITX_SUPER_RCTRL[2];
    extern FcitxHotkey FCITX_LALT[2];
    extern FcitxHotkey FCITX_RALT[2];
    extern FcitxHotkey FCITX_LSUPER[2];
    extern FcitxHotkey FCITX_RSUPER[2];

#ifdef __cplusplus
}
#endif

#endif
// kate: indent-mode cstyle; space-indent on; indent-width 0;
